#include "Arduino.h"
#include <avr/interrupt.h>

// https://github.com/grbl/grbl/blob/658eb6a8b3cff0da59a61cbb5aa04d2c693c36c3/grbl/limits.c

#define EN        8  
//Direction pin
#define X_DIR     5
//Step pin
#define X_STP     2

//DRV8825
//SlowMovement
int delayTimeSlow=10000; //Delay between each pause (uS)
int stepsSlow=64;// Steps to move

//FastMovement For Homing
int delayTimeFast = 50;
int stepsFast=32;
bool home = false;

bool dir = true; //true = to the motor

// Define homing/hard limit switch input pins and limit interrupt vectors. 
// NOTE: All limit bit pins must be on the same port, but not on a port with other input pins (CONTROL).
#define LIMIT_DDR        DDRB
#define LIMIT_PIN        PINB
#define LIMIT_PORT       PORTB
#define X_LIMIT_BIT      1  // Uno Digital Pin 9
#define Y_LIMIT_BIT      2  // Uno Digital Pin 10
#define LIMIT_MASK ((1<<X_LIMIT_BIT)|(1<<Y_LIMIT_BIT)) // All limit bits
#define LIMIT_INT        PCIE0  // Pin change interrupt enable pin
#define LIMIT_INT_vect   PCINT0_vect 
#define LIMIT_PCMSK      PCMSK0 // Pin change interrupt register

#define bit_istrue(x,mask) ((x & mask) != 0)

#define BITFLAG_HARD_LIMIT_ENABLE  bit(3)

void move(boolean dir, byte dirPin, byte stepperPin, int steps, int delayTime)
{
  digitalWrite(dirPin, dir);
  // delay(10);
  for (int i = 0; i < steps; i++) {
    digitalWrite(stepperPin, HIGH);
    delayMicroseconds(delayTime);
    digitalWrite(stepperPin, LOW);
    delayMicroseconds(delayTime);
  }
}

// Disables hard limits.
void limits_disable()
{
  LIMIT_PCMSK &= ~LIMIT_MASK;  // Disable specific pins of the Pin Change Interrupt
  PCICR &= ~(1 << LIMIT_INT);  // Disable Pin Change Interrupt
}

void limits_init() 
{
  LIMIT_DDR &= ~(LIMIT_MASK); // Set as input pins

  LIMIT_PORT |= (LIMIT_MASK);  // Enable internal pull-up resistors. Normal high operation.

	LIMIT_PCMSK |= LIMIT_MASK; // Enable specific pins of the Pin Change Interrupt
	PCICR |= (1 << LIMIT_INT); // Enable Pin Change Interrupt
}

uint8_t limits_get_state()
{
  uint8_t limit_state = 2;

  if (digitalRead(9)==HIGH && digitalRead(10)==LOW){
  	limit_state = 1;
  } 
  if (digitalRead(9) == LOW && digitalRead(10)==HIGH) {
  	limit_state = 0;
  }

  // The first time we home, we don't do it again
  if (!home){
    home = true;
  }
 
  return(limit_state);
}

ISR(LIMIT_INT_vect) // DEFAULT: Limit pin change interrupt process. 
{
// Ignore limit switches if already in an alarm state or in-process of executing an alarm.
// When in the alarm state, Grbl should have been reset or will force a reset, so any pending 
// moves in the planner and serial buffers are all cleared and newly sent blocks will be 
// locked out until a homing cycle or a kill lock command. Allows the user to disable the hard
// limit setting if their limits are constantly triggering after a reset and move their axes.

	// Check limit pin state. 
	// if (limits_get_state() == 1) {
	// 	dir = false;
	// } 
	// if (limits_get_state() == 0) {
	// 	dir = true;
	// }

  uint8_t state = limits_get_state();

  switch (state) {
      case 1:
        dir = false;
        break;
      case 0:
        dir = true;
        break;
  }
}



void setup(){

  //Set the motor Pins
	pinMode(X_DIR, OUTPUT); 
	pinMode(X_STP, OUTPUT);
	pinMode(EN, OUTPUT);
	digitalWrite(EN, LOW);

  //Give some time, before we begin
  delay(5000);

  //Initialise Serial for debugging
	Serial.begin(9600);

  //Initialise registers
	limits_init();
}


void loop(){

  //While not homed, move fast!
  while (home==false) {
    move(dir,X_DIR, X_STP, stepsFast, delayTimeFast);
  }

  //Now move nicely
	move(dir, X_DIR, X_STP, stepsSlow, delayTimeSlow); //X, Clockwise

}